package me.hekr.demo;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

public class GatewayDevice {
  private static final Executor EXECUTOR = Executors.newSingleThreadExecutor();
  // 设备id
  private static final String DEV_TID = "";
  // 产品密钥
  private static final String PROD_KEY = "";
  // 设备私钥
  private static final String PRIV_KEY = "";
  // 网关是否启用2步认证，相关设置在网关产品偏好里面设置
  private static final boolean GATEWAY_DEVICE_TWO_FACTOR_AUTHENTICATION = true;
  // 最大等待时间 毫秒
  private static final int MAX_WAIT_TIME_MILLS = 3000;

  public static void main(String[] args)
      throws IOException, InterruptedException, ExecutionException, TimeoutException {
    TcpClient tcpClient = new TcpClient("hub.hekr.me", 83).connect();

    String randomKey = null;
    Map<String, Object> message;
    Map<String, Object> params;
    // 如果启用两步认证，则需要认证设备信息
    if (GATEWAY_DEVICE_TWO_FACTOR_AUTHENTICATION) {
      // 校验信息
      message = new HashMap<>();
      message.put("action", "requestVerify");
      message.put("msgId", tcpClient.getMsgId());
      params = new HashMap<>();
      params.put("devTid", DEV_TID);
      message.put("params", params);
      tcpClient.send(JSON.toJSONString(message) + "\n");
      String respMsg = tcpClient.receive(MAX_WAIT_TIME_MILLS);
      JSONObject respMap = JSON.parseObject(respMsg);
      boolean success = respMap.getIntValue("code") == 200;
      if (!success) {
        System.out.println("校验失败,resp:" + respMsg);
        return;
      }

      // 获取 randomKey
      randomKey = (String) respMap.get("randomKey");
    }

    // 网关登录
    params = new HashMap<>();
    params.put("devTid", DEV_TID);
    // 如果启用两步认证，需要加上authKey
    if (GATEWAY_DEVICE_TWO_FACTOR_AUTHENTICATION) {
      String authKey = Util.MD5(randomKey + DEV_TID + PRIV_KEY);
      params.put("authKey", authKey);
    }
    message = new HashMap<>();
    message.put("action", "gatewayLogin");
    message.put("msgId", tcpClient.getMsgId());
    message.put("params", params);
    tcpClient.send(JSON.toJSONString(message) + "\n");
    String respMsg = tcpClient.receive(MAX_WAIT_TIME_MILLS);
    JSONObject resp = JSON.parseObject(respMsg);
    if (resp.getIntValue("code") != 200) {
      System.out.println("gateway login failed,resp:" + respMsg);
      return;
    }

    // 登录成功，这里订阅消息
    subcribeMessage(tcpClient);

    // TODO 发送业务数据

    // 循环定时发送心跳
    loopSendHeartbeat(tcpClient);
  }

  private static void loopSendHeartbeat(TcpClient tcpClient) {
    EXECUTOR.execute(
        () -> {
          // 如果client 没关闭，则每20s发送过一次心跳
          while (tcpClient.isOpen()) {
            Map<String, Object> heartbeatMessage = new HashMap<>();
            heartbeatMessage.put("action", "heartbeat");
            heartbeatMessage.put("msgId", tcpClient.getMsgId());
            try {
              tcpClient.send(JSON.toJSONString(heartbeatMessage));
            } catch (IOException e) {
              System.out.println(e.getMessage());
            }
            try {
              TimeUnit.SECONDS.sleep(20);
            } catch (InterruptedException ignored) {
            }
          }
        });
  }

  private static void subcribeMessage(TcpClient tcpClient) {
    EXECUTOR.execute(
        new ReceiveMessageTask(
            tcpClient,
            new MessageListener() {
              @Override
              public void onMessage(String message) {
                System.out.println(message);
              }
            }));
  }
}
