package me.hekr.demo;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/** 描述 Created by du on 2017/8/6. */
public class TcpClient {
  private static final int TIMEOUT_MILL = 20000;
  private Socket client;
  private String host;
  private int port;
  private BufferedReader inFromServer;
  private DataOutputStream outToServer;
  private int msgId;

  public TcpClient(String host, int port) throws SocketException {
    this.host = host;
    this.port = port;
    client = new Socket();
    client.setKeepAlive(true);
  }

  /** 获取msgId，从0-65535 */
  public synchronized int getMsgId() {
    msgId++;
    if (msgId > 65535) {
      msgId = 0;
    }

    return msgId;
  }

  // 建立链接
  public TcpClient connect() throws IOException {
    return connect(30000);
  }

  public TcpClient connect(int timeout) throws IOException {
    client.setKeepAlive(true);
    client.setSoTimeout(TIMEOUT_MILL);
    client.connect(new InetSocketAddress(host, port), timeout);

    outToServer = new DataOutputStream(client.getOutputStream());
    inFromServer = new BufferedReader(new InputStreamReader(client.getInputStream()));
    return this;
  }

  public TcpClient send(String message) throws IOException {
    if (client.isClosed()) {
      throw new RuntimeException("TCP client closed");
    }
    if (client.isOutputShutdown()) {
      throw new RuntimeException("output channel closed");
    }

    outToServer.writeBytes(message);
    return this;
  }

  public void close() {
    try {
      this.client.close();
    } catch (IOException ignored) {
    }
    try {
      if (inFromServer != null) {
        inFromServer.close();
      }
    } catch (IOException ignored) {
    }
    try {
      if (outToServer != null) {
        outToServer.close();
      }
    } catch (IOException ignored) {
    }
  }

  public boolean isOpen() {
    return !client.isClosed();
  }

  /**
   * 获取消息，一直阻塞
   *
   * @return
   * @throws IOException
   */
  public String receive() throws IOException {
    return inFromServer.readLine();
  }

  /**
   * 获取消息
   *
   * @param timeoutMill 超时时间，毫秒
   * @throws IOException
   */
  public String receive(int timeoutMill)
      throws InterruptedException, ExecutionException, TimeoutException {
    return new FutureTask<String>(() -> inFromServer.readLine())
        .get(timeoutMill, TimeUnit.MICROSECONDS);
  }
}
